#!/bin/sh

# Original source:
# Copyright (C) 2012 Amazon.com, Inc. or its affiliates.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#    http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
# OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the
# License.

#
# Copyright (C) 2018 velostrata.com
# All Rights Reserved.
#
get_metadata() {
  key=$1
  header="Metadata-flavor: Google"
  domain="metadata.google.internal"
  prefix="computeMetadata/v1"
  url="http://${domain}/${prefix}/${key}"

  retries=$2
  if [ -z "${retries}" ] ; then
    retries=1
  fi

  while true ; do

    if meta=$(curl -s -f -H "${header}" "${url}") ; then
      echo "${meta}"
      return 0
    fi

    retries=$((retries-1))

    if [ ${retries} -eq 0 ]; then
      break
    fi

    sleep 1
  done
  return 1
}

wait_for_metadata_server() {
  _=$(get_metadata "" 10)
}

should_apply_ip_ranges() {
  apply_key="apply-alias-ip-ranges"
  instance=$(get_metadata "instance/attributes/${apply_key}")
  project=$(get_metadata "project/attributes/${apply_key}")

  if [ "${instance}" = "true" ] ; then
    return 0
  fi
  if [ -n "$instance" ] ; then
    return 1
  fi
  if [ "$project" = "true" ] ; then
    return 0
  fi
  return 1
}

is_suse() {
  if [ -f "/etc/SuSE-release" ]; then
    return 0
  fi
  if [ -f "/etc/os-release" ] ; then
    if [ "$(. /etc/os-release && echo "${ID}")" = "suse" ] ; then
      return 0
    fi
    if [ "$(. /etc/os-release && echo "${ID_LIKE}")" = "suse" ] ; then
      return 0
    fi
  fi
  return 1
}

# shellcheck disable=SC2154
# Given by dhclient
if [ "${reason}" = "PREINIT" ] ; then
  # Network not ready yet - can't check metadata
  exit 0
fi

wait_for_metadata_server

if should_apply_ip_ranges ; then

  if grep -q Ubuntu /etc/issue || grep -q Debian /etc/issue; then
    if [ -d /etc/netplan ] ; then
      interface=$(ls /sys/class/net/ | grep -v lo | head -n 1)
      new_prefix=$(/sbin/ip add show ${interface} | grep "inet " | awk '{print $2}' | cut -d "/" -f 2)
    fi
    /bin/bash -c "INTERFACE=${interface}; \
      PREFIX=${new_prefix}; \
      . /etc/migrate-for-gce/alias-ips/alias-ip-functions; \
      rewrite_rules; \
      rewrite_aliases"
  elif is_suse ; then
    interface="$1"
    if [ "${interface}" != "lo" ]; then
      shift
      INTERFACE="${interface}"
      PREFIX=$(/sbin/ip add show ${INTERFACE} | grep "inet " | awk '{print $2}' | cut -d "/" -f 2)
      . /etc/migrate-for-gce/alias-ips/alias-ip-functions
      rewrite_rules
      rewrite_aliases
    fi
  else
    INTERFACE="${interface}"
    PREFIX="${new_prefix}"
    . /etc/migrate-for-gce/alias-ips/alias-ip-functions

    aliasip_config() {
      rewrite_rules
      rewrite_aliases
    }

    aliasip_restore() {
      remove_aliases
      remove_rules
    }
  fi
fi