#!/bin/sh

TTYS0_CONSOLE_PREFIX="console=ttyS0"
TTYS0_CONSOLE_VALUE="${TTYS0_CONSOLE_PREFIX},115200n8"

FLAGS_TO_ADD="
velostrata
nosplash
mount.timeout=180s
${TTYS0_CONSOLE_VALUE}
"

FLAGS_TO_REMOVE="
debug
quiet
"

log() {
  echo "migrate-for-gce: grub2: $*" >&2
}

remove_bad_ttys0 () {
  # Remove ttyS0 with bad baudrate
  cmdline="$1"
  new_cmdline=""
  for value in ${cmdline} ; do
    case "${value}" in
      "${TTYS0_CONSOLE_VALUE}")
        ;;
      "${TTYS0_CONSOLE_PREFIX}"*)
        # != ${TTYS0_CONSOLE_VALUE}
        log "Removing ttyS0 kernel option: ${value}"
        continue
        ;;
      *)
        ;;
    esac
    new_cmdline="${new_cmdline} ${value}"
  done
  echo "${new_cmdline}"
}

remove_flags() {
  cmdline="$1"
  new_cmdline=""
  for value in ${cmdline} ; do
    remove=
    for to_remove in ${FLAGS_TO_REMOVE} ; do
      if [ "${value}" = "${to_remove}" ] ; then
        remove="y"
        break
      fi
    done
    if [ "${remove}" = "y" ] ; then
      log "Removing kernel option: ${value}"
      continue
    fi
    new_cmdline="${new_cmdline} ${value}"
  done
  echo "${new_cmdline}"
}

add_flags() {
  cmdline="$1"
  cmdline_default="$2"

  full_cmdline="${cmdline} ${cmdline_default}"

  flags_to_add=""

  for to_add in ${FLAGS_TO_ADD} ; do
    exists=
    for value in ${full_cmdline} ; do
      if [ "${value}" = "${to_add}" ] ; then
        exists="y"
        break
      fi
    done
    if [ "${exists}" = "y" ] ; then
      continue
    fi
    log "Adding kernel option: ${to_add}"
    flags_to_add="${flags_to_add} ${to_add}"
  done
  echo "${cmdline} ${flags_to_add}"
}

GRUB_CMDLINE_LINUX=$(remove_bad_ttys0 "${GRUB_CMDLINE_LINUX}")
GRUB_CMDLINE_LINUX_DEFAULT=$(remove_bad_ttys0 "${GRUB_CMDLINE_LINUX_DEFAULT}")

GRUB_CMDLINE_LINUX=$(remove_flags "${GRUB_CMDLINE_LINUX}")
GRUB_CMDLINE_LINUX_DEFAULT=$(remove_flags "${GRUB_CMDLINE_LINUX_DEFAULT}")

GRUB_CMDLINE_LINUX=$(add_flags "${GRUB_CMDLINE_LINUX}")
