#!/bin/sh

netcfg=/etc/network/interfaces
netcfg_backup=/etc/network/interfaces.pre-gce

command_exists() {
  command -v "$1" >/dev/null 2>&1
}

get_nic_interfaces() {
  for device in /sys/class/net/* ; do
    if [ ! -d "${device}" ] ; then
      continue
    fi
    device="${device##*/}"
    # Filter out devices that do not start with "en" or "eth"
    case "${device}" in
      "en"*) ;;
      "eth"*) ;;
      *) continue
    esac
    echo "${device}"
  done
}

# On netplan systems such as Ubuntu 18 - ifupdown is not installed and
# /etc/network/interfaces is simply ignored - no need to fix the contents of
# this file
if ! command_exists ifup ; then
  exit 0
fi

if [ "$1" = "revert" ] ; then
  if [ ! -f ${netcfg_backup} ] ; then
    exit 0
  fi
  if ! mv -f ${netcfg_backup} ${netcfg} ; then
    exit $? # mv exit status
  fi
  # Handle interfaces configured as "allow-hotplug" in original interfaces
  # file.
  # udev invokes ifupdown-hotplug on these interfaces early in the boot
  # process (See /lib/udev/rules.d/80-ifupdown.rules).
  # Manually invoking ifupdown-hotplug on the interfaces in the original
  # interfaces file.
  rc=0
  if [ -f /lib/udev/ifupdown-hotplug ] ; then
    for interface in $(ifquery --allow hotplug -l) ; do
      if ! ACTION=add INTERFACE=$interface /lib/udev/ifupdown-hotplug ; then
        rc=$?
      fi
    done
  fi
  exit $rc
elif [ "$1" = "detach" ] ; then
  interface=""
  for device in $(get_nic_interfaces) ; do
    interface="${device}"
    break
  done
elif [ "$1" = "iscsi" ] ; then
  interface=""
else
  # Bad option
  exit 1
fi

# During iscsi boot and detach - backup original file
# (Only if not already backed up)
if [ ! -f ${netcfg_backup} ] ; then
  cp ${netcfg} ${netcfg_backup}
fi

{
  echo "# Added for Google Cloud by Migrate for Compute Engine"
  echo ""
  echo "auto lo"
  echo "iface lo inet loopback"
  echo ""
  if [ -n "${interface}" ] ; then
    echo "auto ${interface}"
    echo "iface ${interface} inet dhcp"
    echo
  fi
} > ${netcfg}

exit 0
