#!/bin/sh

netplan_dir="/etc/netplan"
# Use 51 to have a higher priority than 51-cloud-init
netplan_new_yaml="${netplan_dir}/51-google-cloud.yaml"

command_exists() {
  command -v "$1" >/dev/null 2>&1
}

get_nic_interfaces() {
  for device in /sys/class/net/* ; do
    if [ ! -d "${device}" ] ; then
      continue
    fi
    device="${device##*/}"
    # Filter out devices that do not start with "en" or "eth"
    case "${device}" in
      "en"*) ;;
      "eth"*) ;;
      *) continue
    esac
    echo "${device}"
  done
}

if [ ! -d "${netplan_dir}" ] ; then
  exit 0
fi

if ! command_exists netplan ; then
  exit 0
fi

if [ "$1" = "revert" ] ; then
  # No need to revert
  exit 0
elif [ "$1" = "iscsi" ] ; then
  # iscsi boot works on netplan systems when ibft0 is not
  # configured properly and netplan files are configured for the previous
  # non-existant interface
  exit 0
elif [ "$1" = "detach" ] ; then
  : # Okay
else
  # Bad option
  exit 1
fi

interface=""
for device in $(get_nic_interfaces) ; do
  interface="${device}"
  # Support a single interface only
  break
done
if [ -z "${interface}" ] ; then
  # Could not find interface
  exit 1
fi

# Backup yaml files
for yaml in /etc/netplan/*.yaml ; do
  if [ ! -f "${yaml}" ] ; then
    continue
  fi
  # Non .yaml files are ignored by netplan
  mv -f "${yaml}" "${yaml}.pre-gce"
done

{
  echo "# Added for Google Cloud by Migrate for Compute Engine"
  echo ""
  echo "network:"
  echo "  version: 2"
  echo "  renderer: networkd"
  echo "  ethernets:"
  echo "    ${interface}:"
  echo "      dhcp4: yes"
} > ${netplan_new_yaml}

# Invoke netplan apply to generatate the network configuration files for its
# backend.
# For "networkd" files are located under /run/systemd/network
netplan apply

exit 0
