#!/bin/bash

[ -n "${TEST}" ] || . /opt/migrate-for-gce/functions.sh

sshd_conf=/etc/ssh/sshd_config
sshd_conf_backup=/etc/ssh/sshd_config.pre-gce

first_line_of() {
  grep -nm 1 "^[[:space:]]*$1" "${sshd_conf}" | cut -d: -f1
}

banner="# Added for Google Cloud by Migrate for Compute Engine"

ensure_key_value() {
  key=$1
  value=$2

  first_match_line=$(first_line_of "Match")
  first_key_line=$(first_line_of "${key}")

  # Change existing values
  sed -i "s|^\(\s*\)${key}.*|\1${key} ${value}|g" "${sshd_conf}"

  # Add "key value" (before Match clause if exists)
  if [ -z "${first_key_line}" ] ; then
    if [ -z "${first_match_line}" ] ; then
      # Add at the end
      echo "${banner}" >> "${sshd_conf}"
      echo "${key} ${value}" >> "${sshd_conf}"
    else
      sed -i "${first_match_line}i${banner}" "${sshd_conf}"
      first_match_line=$((first_match_line+1))
      sed -i "${first_match_line}i${key} ${value}" "${sshd_conf}"
    fi
  elif [ -n "${first_match_line}" ] &&
       [ "${first_key_line}" -gt "${first_match_line}" ] ; then
    sed -i "${first_match_line}i${banner}" "${sshd_conf}"
    first_match_line=$((first_match_line+1))
    sed -i "${first_match_line}i${key} ${value}" "${sshd_conf}"
  fi

}

ensure_pubkey_authentication() {
  ensure_key_value "PubkeyAuthentication" "yes"
}

ensure_authorized_keys_file() {
  ensure_key_value "AuthorizedKeysFile" \
                   ".ssh/authorized_keys"
}

fix_sshd_config() {
  if [ ! -f "${sshd_conf}" ] ; then
    migrate_log "${sshd_conf} does not exist - skipping"
    return 0
  fi
  if [ -f "${sshd_conf_backup}" ] ; then
    migrate_log "${sshd_conf_backup} already exists - skipping"
    return 0
  fi
  migrate_log "Backing up ${sshd_conf} as ${sshd_conf_backup}"
  cp -f "${sshd_conf}" "${sshd_conf_backup}"
  ensure_pubkey_authentication
  ensure_authorized_keys_file
  migrate_log "Fixed ${sshd_conf}"
  return 0
}

revert_sshd_config() {
  if [ ! -f "${sshd_conf_backup}" ] ; then
    migrate_log "${sshd_conf_backup} does not exist - skipping"
    return 0
  fi
  mv -f "${sshd_conf_backup}" "${sshd_conf}"
  return $?
}

if [ -n "${TEST}" ] ; then
  # Allow sourcing this file
  return 0
fi

if [ "$1" = "revert" ] ; then
  revert_sshd_config
  exit $?
fi

fix_sshd_config

exit $?
