#!/bin/bash

source /opt/migrate-for-gce/functions.sh


# Check for direct mounts
# $1 : grub file to check
#
function check_direct_boot_path() {
	if find_active_str $1 "root=/dev/sd" "root=/dev/hd" "root=/dev/.*cciss" "root=/dev/.*mpath" "root=/dev/dm-" ; then
		return 0
	fi

	return 1
}


# Update boot entry in GRUB1 files
# $1 : file
# $2 : device/partition
# $3 : UUID
#
function replace_grub_entry {
	local dev uuid
	dev=$2
	uuid=$3

	if find_active_str $1 "${dev}"; then
		migrate_log "Replacing direct boot mapping ${dev} with: UUID=${uuid}"
		replace_entry $1 "root=${dev}" "root=UUID=${uuid}"
		replace_entry $1 "resume=${dev}" "resume=UUID=${uuid}"
	fi
}

# Get direct device map using blkid
# Output only relevant devices for direct mappings (see is_direct_dev)
# Output format:
# DEV1 UUID1
# DEV2 UUID2
#
function get_direct_mappings()
{
	while read dev ; do
		uuid=$(blkid -c /dev/null -o value -s UUID $dev)
		if is_direct_dev ${dev}; then
			echo $dev $uuid
		fi
	done < <(blkid -c /dev/null -o device)
}


# GRUB config files
grub_files=${DEFAULT_GRUB_CONFIG_FILES[*]}
while [ "$1" != "" ]; do
	grub_files+=($1)
	shift
done
migrate_log Checking GRUB config: ${grub_files[*]}

for ff in ${grub_files[*]}; do
	if is_real_file ${ff} && check_direct_boot_path ${ff}; then
		backup_file=${ff}.$$
		migrate_log "Direct mounts found in ${ff}, saving backup copy to ${backup_file}"
		cp -f ${ff} ${backup_file}

		while read dev uuid ; do
			replace_grub_entry ${ff} ${dev} ${uuid}
		done < <(get_direct_mappings)
	fi
done

# /etc/fstab
conf=/etc/fstab
backup_file=/etc/fstab.pre-gce_$$
resolve_fstab_entries ${conf} ${conf}.TMP
modified=$?
if [ ${modified} -ne 0 ] ; then
	migrate_log "Direct mounts found in ${conf}, saved backup copy to ${backup_file}"
	cp -f ${conf} ${backup_file}
	mv -f /etc/fstab.TMP /etc/fstab
else
	rm -f /etc/fstab.TMP
fi

# LVM configuration
conf=/etc/lvm/lvm.conf
update_lvm_filter ${conf} ${conf}.TMP '[ "a/.*/" ]'
modified=$?
if [ ${modified} -ne 0 ] ; then
	migrate_log "Device-specific LVM filters found in ${conf}, saving backup copy to ${conf}.pre-gce_$$"
	cp -f ${conf} ${conf}.pre-gce_$$
	mv -f ${conf}.TMP ${conf}
else
	rm -f ${conf}.TMP
fi

